function extras = gtFigureCreateExtras(type, varargin)
    switch (type)
        case 'unit_bar'
            extras = create_unit_bar(varargin{:});
        case {'line_vertical', 'line_horizontal'}
            extras = create_line(varargin{:});
        case 'box'
            extras = create_box(varargin{:});
        case 'point'
            extras = create_point(varargin{:});
        case 'text'
            extras = create_text(varargin{:});
        case 'arrow'
            extras = create_arrow(varargin{:});
        case 'segment'
            extras = create_segment(varargin{:});
        case 'inset_zoom'
            extras = create_inset_zoom(varargin{:});
        case 'inset_image'
            extras = create_inset_image(varargin{:});
    end
    extras.type = type;
end

function extras = create_unit_bar(varargin)
    extras = struct( ...
        'base_unit', 'um', ... % µm (\u00B5m), \mu m, mm
        'line_length', 10, ...
        'line_width', 2, ...
        'position', 'south_west', ... {'north_west', 'south_west', 'north_est', 'south_est'}
        'margin', [10 10], ...
        'font_size', 10, ...
        'font_weight', 'normal', ... {'normal', 'bold', 'light', 'demi'}
        'font_angle', 'normal', ... {'normal', 'italic', 'oblique'}
        'color', [1 1 1] );
    extras = parse_pv_pairs(extras, varargin);
end

function extras = create_line(varargin)
    extras = struct( ...
        'line_width', 2, ...
        'line_style', '-', ...
        'position', 1, ...
        'color', [1 1 1] );
    extras = parse_pv_pairs(extras, varargin);
end

function extras = create_box(varargin)
    extras = struct( ...
        'line_width', 2, ...
        'line_style', '-', ...
        'position', [1 1 10 10], ...
        'color', [1 1 1] );
    extras = parse_pv_pairs(extras, varargin);
end

function extras = create_point(varargin)
    extras = struct( ...
        'size', 20, ...
        'style', 'o', ...
        'position', [1 1], ...
        'color', [1 1 1] );
    extras = parse_pv_pairs(extras, varargin);
end

function extras = create_text(varargin)
    extras = struct( ...
        'string', '', ...
        'position', [1 1], ...
        'alignment_horz', 'center', ... {'left', 'center', 'right'}
        'alignment_vert', 'middle', ... {'top', 'cap', 'middle', 'baseline', 'bottom'}
        'font_size', 10, ...
        'font_weight', 'normal', ... {'normal', 'bold', 'light', 'demi'}
        'font_angle', 'normal', ... {'normal', 'italic', 'oblique'}
        'color', [1 1 1], ...
        'interpreter', 'tex' );
    extras = parse_pv_pairs(extras, varargin);
end

function extras = create_arrow(varargin)
    extras = struct( ...
        'line_length', 10, ...
        'line_width', 0.5, ...
        'line_style', '-', ...
        'position', [1 1], ...
        'direction', [1 1], ...
        'color', [1 1 1] );
    extras = parse_pv_pairs(extras, varargin);
end

function extras = create_segment(varargin)
    extras = struct( ...
        'line_width', 1, ...
        'line_style', '-', ...
        'position', [1 1 10 10], ...
        'color', [1 1 1] );
    extras = parse_pv_pairs(extras, varargin);
end

function extras = create_inset_zoom(varargin)
    extras = struct( ...
        'line_width', 1, ...
        'line_style', '-', ...
        'region', [1 1 10 10], ...
        'position', [1 1 10 10], ...
        'color', [1 1 1] );
    extras = parse_pv_pairs(extras, varargin);
end

function extras = create_inset_image(varargin)
    extras = struct( ...
        'line_width', 1, ...
        'line_style', '-', ...
        'image', [], ...
        'position', [1 1 10 10], ...
        'color', [1 1 1] );
    extras = parse_pv_pairs(extras, varargin);
end

